@@ -27,6 +27,12 @@ public:
     void setLocked(bool lock);
     bool isLocked() const;
 
+protected:
+    /**
+     * Make sure we do not emit QDockWidget::visibilityChanged() signals whenever Dolphin's window is minimized or restored.
+     */
+    bool event(QEvent *event) override;
+
 private:
     bool m_locked;
     QWidget *m_dockTitleBar;
