@@ -32,6 +32,7 @@
 
 typedef KIO::FileUndoManager::CommandType CommandType;
 
+class DiskSpaceUsageMenu;
 class DolphinBookmarkHandler;
 class DolphinViewActionHandler;
 class DolphinSettingsDialog;
@@ -704,6 +705,12 @@ private:
      */
     void updateAllowedToolbarAreas();
 
+    /**
+     * Sets the background of the location bar(s) on if they are inside the toolbar,
+     * or off if they are outside the toolbar.
+     */
+    void updateNavigatorsBackground();
+
     bool isKompareInstalled() const;
 
     /**
@@ -766,6 +773,7 @@ private:
     KToolBarPopupAction *m_forwardAction;
     KActionMenu *m_splitViewAction;
     QAction *m_splitViewMenuAction;
+    DiskSpaceUsageMenu *m_diskSpaceUsageMenu;
 
     QMenu m_searchTools;
     KFileItemActions m_fileItemActions;
