@@ -30,7 +30,7 @@ QList<QAction *> MoveToNewFolderItemAction::actions(const KFileItemListPropertie
 
     QAction *createFolderFromSelected = new QAction(i18nc("@action:inmenu", "Move to New Folder…"), parentWidget);
     createFolderFromSelected->setIcon(QIcon::fromTheme(QStringLiteral("folder-new")));
-    connect(createFolderFromSelected, &QAction::triggered, this, [=]() {
+    connect(createFolderFromSelected, &QAction::triggered, this, [=, this]() {
         QString selectedFileDirPath = selectedItems.at(0).url().toString().remove(selectedItems.at(0).name());
         if (selectedFileDirPath.endsWith(QStringLiteral("/"))) {
             selectedFileDirPath.removeLast();
