@@ -340,6 +340,10 @@ private:
 
     void trimHoverSequenceLoadedItems();
 
+    void resetSizeData(const int index, const int size = 0);
+
+    void recountDirectoryItems(const QList<QUrl> directories);
+
 private:
     QSize cacheSize();
     /**
