@@ -154,6 +154,12 @@ public:
         return m_columnCount;
     }
 
+    /**
+     * Set the bottom offset for moving the view so that the small overlayed statusbar
+     * won't cover any items by accident.
+     */
+    void setStatusBarOffset(int offset);
+
 #ifndef QT_NO_DEBUG
     /**
      * @return True if the layouter has been marked as dirty and hence has
@@ -219,6 +225,8 @@ private:
     };
     QVector<ItemInfo> m_itemInfos;
 
+    int m_statusBarOffset;
+
     friend class KItemListControllerTest;
 };
 
