@@ -25,7 +25,7 @@ class ServiceModel : public QAbstractListModel
     Q_OBJECT
 
 public:
-    enum Role { DesktopEntryNameRole = Qt::UserRole, ConfigurableRole };
+    enum Role { DesktopEntryNameRole = Qt::UserRole };
 
     explicit ServiceModel(QObject *parent = nullptr);
     ~ServiceModel() override;
@@ -35,11 +35,11 @@ public:
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     void clear();
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
 
 private:
     struct ServiceItem {
-        bool checked;
-        bool configurable;
+        Qt::CheckState checked;
         QString icon;
         QString text;
         QString desktopEntryName;
